#
# Sensus Consulting Ltd. (C) 1997-1998
# Matt Newman <matt@sensus.org>
#
# Generic Tree
#
#INDEX\
proc tcl++::tree {} {}

package require tcl++

proc tree {{this #auto} args} {uplevel 1 tcl++::tree $this $args}

class tcl++::tree {
    inherit tcl++::atree

    public {
		constructor {args} {}

		method add {id args}
		method delete {id}
		method get {id args}
		method put {id fld val args}

		method children {id {order -increasing}}
		method parent {id}
    }
    protected {
		variable uid 0
		variable store
    }
}

body tcl++::tree::constructor {args} {
	set id 0

	set store($id) [list id $id name $id class index]
	set store(${id},parent)		$id
	set store(${id},children)	{}

	if {[llength $args]>0} {
		set store($id) [eval pset [list $store($id)] $args]
	}
}
#
# Get & Set attributes of specific nodes.
#
body tcl++::tree::get {id args} {
	switch -- [llength $args] {
	0	{return $store($id)}
	1	-
	2	{return [eval [list pget $store($id)] $args]}
	default	{
		error "wrong # of args: should be \"get id ?fld ?default??\""
	}
	};# switch
}
body tcl++::tree::put {id fld val args} {
	set store($id) [eval pset [list $store($id) $fld $val] $args]
}
body tcl++::tree::children {id {order -increasing}} {
	return $store(${id},children)
}
body tcl++::tree::parent {id} {
	return $store(${id},parent)
}
body tcl++::tree::add {id args} {
	set cid [incr uid]

	lappend store(${id},children) $cid

	set store($cid) [list id $cid name "" class node]
	set store(${cid},parent)	$id
	set store(${cid},children)	{}

	if {[llength $args]>0} {
		set store($cid) [eval pset [list $store($cid)] $args]
	}
	return $cid
}
body tcl++::tree::delete {id} {
	foreach cid $store(${id},children) {
		delete $cid
	}
	set pid $store(${id},parent)
	set idx [lsearch $store(${pid},children) $id]
	set store(${pid},children) [lreplace $store(${pid},children) $idx $idx]

	unset store($id)
}
