#
# Sensus Consulting Ltd (C) 1997
# Matt Newman <matt@sensus.org>
#
# Base Class for lightweight canvas compound objects
#
#INDEX\
proc tk++::Canvas::Label {} {}

package require tk++

proc clabel {args} {return [eval tk++::Canvas::Label $args]}

class tk++::Canvas::Label {
	inherit tk++::Canvas::Object

	constructor {args} {eval configure $args}

	public {
		method coords {tag args} {}
		method create {type x y args} {}
	}
	protected {
		method refresh {tag x y} {}
	}
	common options
	array set options {
		-anchor		center
		-font		{{MS Sans Serif} 8}
		-foreground black
		-background	{}
		-justify	left
		-text		{}
		-state		{}
		-width		0
		-height		0
		-borderwidth	0
	}
}
body tk++::Canvas::Label::coords {tag args} {
}
body tk++::Canvas::Label::create {type x y args} {
	variable options

	array set opts [array get options]
	array set opts $args

	set tag [chain $type $x $y]

	set tags($tag,text) [$canvas create text 0 0 -tags $tag]
	set tags($tag,rect) [$canvas create rect $x $y \
						[expr {$x + $opts(-width)}] \
						[expr {$y + $opts(-height)}] \
						-tags $tag]
	$canvas raise $tags($tag,text) $tags($tag,rect)

	eval itemconfigure $tag [array get opts]

	return $tag
}
body tk++::Canvas::Label::refresh {tag x y} {
	set width	[pget $items($tag) -width]
	set height	[pget $items($tag) -height]
	set font	[pget $items($tag) -font]
	set text	[pget $items($tag) -text]
	set bw		[pget $items($tag) -borderwidth]
	# Compute font info
	array set fn	[font metrics $font]
	set reqw	[font measure $font $text]
	set reqh	[expr {$fn(-ascent) + $fn(-descent)}]
	if {$width == 0} {
		# Auto-size
		set width $reqw
	} else {
		# Munge text to fit
		if {[font measure $font "${text}"] > $width} {
			set tlen [string length $text]
			while {$tlen > -1 && [font measure $font "${text}..."] >= $width} {
				set text [string range $text 0 $tlen]
				incr tlen -1
			}
			set text "${text}..."
		}
	}
	if {$height < $reqh} {
		# Auto-size
		set height $reqh
	}
	if {$bw  == 0} {
		set outline {}
	} else {
		set outline black
	}
	$canvas itemconfigure $tags($tag,text) \
			-font $font -width $width -text $text \
			-fill [pget $items($tag) -foreground] \
			-anchor [pget $items($tag) -anchor] \
			-justify [pget $items($tag) -justify]

	$canvas itemconfigure $tags($tag,rect) \
			-width $bw -outline $outline \
			-fill [pget $items($tag) -background]

	# Position & Size rectangle
	$canvas coords $tags($tag,rect) \
					$x $y \
					[expr {$x + 2*$bw + $width}] [expr {$y + 2*$bw + $height}]
	# Position & Size text
	switch -glob -- [pget $items($tag) -anchor] {
	ne	-
	e	-
	se	{set x [expr {$x + $bw + $width}]}
	nw	-
	w	-
	sw	{set x [expr {$x + $bw}]}
	n	-
	c*	-
	s	{set x [expr {$x + $bw + $width/2.0}]}
	}
	switch -glob -- [pget $items($tag) -anchor] {
	nw	-
	n	-
	ne	{set y [expr {$y + $bw}]}
	w	-
	c*	-
	e	{set y [expr {$y + $bw + $height/2.0}]}
	sw	-
	s	-
	se	{set y [expr {$y + $bw + $height}]}
	}
	$canvas coords $tags($tag,text) $x $y
}
